@@ -53,7 +53,21 @@ class Annotation(es.Model):
 
         return q
 
-    def save(self):
+    @classmethod
+    def _build_query_raw(cls, request):
+        q, p = super(Annotation, cls)._build_query_raw(request)
+
+        if current_app.config.get('AUTHZ_ON'):
+            f = authz.permissions_filter(g.user)
+            if not f:
+                return {'error': "Authorization error!", 'status': 400}, None
+            if 'query' in q:
+                q['query'] = {'filtered': {'query': q['query'],
+                                           'filter': f}}
+
+        return q, p
+
+    def save(self, *args, **kwargs):
         # For brand new annotations
         _add_created(self)
         _add_default_permissions(self)
@@ -61,7 +75,7 @@ class Annotation(es.Model):
         # For all annotations about to be saved
         _add_updated(self)
 
-        super(Annotation, self).save()
+        super(Annotation, self).save(*args, **kwargs)
 
 
 def _add_created(ann):
