@@ -3,7 +3,9 @@ import json
 from flask import Blueprint, Response
 from flask import g
 from flask import request
+from flask import url_for
 
+from annotator.atoi import atoi
 from annotator.annotation import Annotation
 
 store = Blueprint('store', __name__)
@@ -42,7 +44,55 @@ def after_request(response):
 # ROOT
 @store.route('/')
 def root():
-    return jsonify("Annotator Store API")
+    return jsonify({
+        'message': "Annotator Store API",
+        'links': {
+            'annotation': {
+                'create': {
+                    'method': 'POST',
+                    'url': url_for('.create_annotation', _external=True),
+                    'query': {
+                        'refresh': {
+                            'type': 'bool',
+                            'desc': "Force an index refresh after create (default: true)"
+                        }
+                    },
+                    'desc': "Create a new annotation"
+                },
+                'read': {
+                    'method': 'GET',
+                    'url': url_for('.read_annotation', id=':id', _external=True),
+                    'desc': "Get an existing annotation"
+                },
+                'update': {
+                    'method': 'PUT',
+                    'url': url_for('.update_annotation', id=':id', _external=True),
+                    'query': {
+                        'refresh': {
+                            'type': 'bool',
+                            'desc': "Force an index refresh after update (default: true)"
+                        }
+                    },
+                    'desc': "Update an existing annotation"
+                },
+                'delete': {
+                    'method': 'DELETE',
+                    'url': url_for('.delete_annotation', id=':id', _external=True),
+                    'desc': "Delete an annotation"
+                }
+            },
+            'search': {
+                'method': 'GET',
+                'url': url_for('.search_annotations', _external=True),
+                'desc': 'Basic search API'
+            },
+            'search_raw': {
+                'method': 'GET/POST',
+                'url': url_for('.search_annotations_raw', _external=True),
+                'desc': 'Advanced search API -- direct access to ElasticSearch. Uses the same API as the ElasticSearch query endpoint.'
+            }
+        }
+    })
 
 # INDEX
 @store.route('/annotations')
@@ -57,18 +107,19 @@ def create_annotation():
     if g.user is None:
         return _failed_authz_response('create annotation')
 
-    if request.json:
+    if request.json is not None:
         annotation = Annotation(_filter_input(request.json, CREATE_FILTER_FIELDS))
 
         annotation['consumer'] = g.user.consumer.key
         if _get_annotation_user(annotation) != g.user.id:
             annotation['user'] = g.user.id
 
-        annotation.save()
-
         if hasattr(g, 'after_annotation_create'):
+            annotation.save(refresh=False)
             g.after_annotation_create(annotation)
-            annotation.save()
+
+        refresh = request.args.get('refresh') != 'false'
+        annotation.save(refresh=refresh)
 
         return jsonify(annotation)
     else:
@@ -98,7 +149,7 @@ def update_annotation(id):
     if failure:
         return failure
 
-    if request.json:
+    if request.json is not None:
         updated = _filter_input(request.json, UPDATE_FILTER_FIELDS)
         updated['id'] = id # use id from URL, regardless of what arrives in JSON payload
 
@@ -112,7 +163,8 @@ def update_annotation(id):
         if hasattr(g, 'before_annotation_update'):
             g.before_annotation_update(annotation)
 
-        annotation.save()
+        refresh = request.args.get('refresh') != 'false'
+        annotation.save(refresh=refresh)
 
     return jsonify(annotation)
 
@@ -137,9 +189,9 @@ def search_annotations():
     kwargs = dict(request.args.items())
 
     if 'offset' in kwargs:
-        kwargs['offset'] = _quiet_int(kwargs['offset'])
+        kwargs['offset'] = atoi(kwargs['offset'])
     if 'limit' in kwargs:
-        kwargs['limit'] = _quiet_int(kwargs['limit'], 20)
+        kwargs['limit'] = atoi(kwargs['limit'], 20)
 
     results = Annotation.search(**kwargs)
     total = Annotation.count(**kwargs)
@@ -148,6 +200,12 @@ def search_annotations():
         'rows': results
     })
 
+# RAW ES SEARCH
+@store.route('/search_raw', methods=['GET', 'POST'])
+def search_annotations_raw():
+    res = Annotation.search_raw(request)
+    return jsonify(res, status=res.get('status', 200))
+
 def _filter_input(obj, fields):
     for field in fields:
         obj.pop(field, None)
@@ -177,9 +235,3 @@ def _failed_authz_response(msg=''):
                    "a user with appropriate permissions on this annotation? "
                    "(user={user}, consumer={consumer})".format(' (' + msg + ')' if msg else '', user=user, consumer=consumer),
                    status=401)
-
-def _quiet_int(obj, default=0):
-    try:
-        return int(obj)
-    except ValueError:
-        return default
